#!/usr/bin/python
# Sview image gallery by Damien Tanner (dctanner@magenet.com)      
#
#    Sview is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    Sview is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Sview; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from gtk import *
import gdkpixbuf
import os

class Sview:
	def __init__(self):
		self.VERSION = '0.2.0'
		self.view_type = 'list'
		self.images_dir = ''
		self.image_scale = 'scaled'
		self.slideshow_on = 0
		self.window = GtkWindow(WINDOW_TOPLEVEL)
		self.window.connect("delete_event", self.quit)
		self.window.set_title("Sview " + self.VERSION)
		if screen_width() > 300:
			self.window.set_usize(280, 240)
		else:
			self.window.set_usize(240, 280)
		self.window.show()

		self.vbox = GtkVBox()
		self.window.add(self.vbox)
		self.vbox.show()

		hbox = GtkHBox()
		self.vbox.pack_start(hbox, expand=FALSE)
		hbox.show()

		self.toolbar1 = GtkToolbar(ORIENTATION_HORIZONTAL, TOOLBAR_ICONS)
		self.toolbar1.set_button_relief(RELIEF_NONE)
		self.toolbar1.set_space_style(TOOLBAR_SPACE_LINE)
		self.toolbar1.set_space_size(10)
		hbox.pack_start(self.toolbar1, expand=FALSE)
		self.toolbar1.show()

		pix, mask = create_pixmap_from_xpm(self.window, None, "/usr/share/sview/pixmaps/open.xpm")
		self.toolbar1.append_item(None, 'Open image directory', None, GtkPixmap(pix, mask), self.select_dir)

		self.toolbar1.append_space()

		self.toolbar2 = GtkToolbar(ORIENTATION_HORIZONTAL, TOOLBAR_ICONS)
		self.toolbar2.set_button_relief(RELIEF_NONE)
		self.toolbar2.set_space_style(TOOLBAR_SPACE_LINE)
		self.toolbar2.set_space_size(10)
		self.toolbar2.set_sensitive(FALSE)
		hbox.pack_start(self.toolbar2, expand=FALSE)
		self.toolbar2.show()

		pix, mask = create_pixmap_from_xpm(self.window, None, "/usr/share/sview/pixmaps/left.xpm")
		self.toolbar2.append_item(None, 'Previous image', None, GtkPixmap(pix, mask), self.prev_image)

		pix, mask = create_pixmap_from_xpm(self.window, None, "/usr/share/sview/pixmaps/right.xpm")
		self.toolbar2.append_item(None, 'Next image', None, GtkPixmap(pix, mask), self.next_image)

		pix, mask = create_pixmap_from_xpm(self.window, None, "/usr/share/sview/pixmaps/fullscreen.xpm")
		self.toolbar2.append_item(None, 'Fullscreen', None, GtkPixmap(pix, mask), self.scale_image)

		pix, mask = create_pixmap_from_xpm(self.window, None, "/usr/share/sview/pixmaps/close.xpm")
		self.toolbar2.append_item(None, 'Close image', None, GtkPixmap(pix, mask), self.show_main)

		self.toolbar3 = GtkToolbar(ORIENTATION_HORIZONTAL, TOOLBAR_ICONS)
		self.toolbar3.set_button_relief(RELIEF_NONE)
		self.toolbar3.set_space_style(TOOLBAR_SPACE_LINE)
		self.toolbar3.set_space_size(10)
		self.toolbar3.set_sensitive(FALSE)
		hbox.pack_start(self.toolbar3, expand=TRUE)
		self.toolbar3.show()

		self.toolbar3.append_space()

		pix, mask = create_pixmap_from_xpm(self.window, None, "/usr/share/sview/pixmaps/list.xpm")
		self.toolbar3.append_item(None, 'Extended list view', None, GtkPixmap(pix, mask), self.show_list_view)

		pix, mask = create_pixmap_from_xpm(self.window, None, "/usr/share/sview/pixmaps/thumbs.xpm")
		self.toolbar3.append_item(None, 'Tumbnails view', None, GtkPixmap(pix, mask), self.show_thumbs_view)

		pix, mask = create_pixmap_from_xpm(self.window, None, "/usr/share/sview/pixmaps/refresh.xpm")
		self.toolbar3.append_item(None, 'Refresh images', None, GtkPixmap(pix, mask), self.refresh_dir)

		pix, mask = create_pixmap_from_xpm(self.window, None, "/usr/share/sview/pixmaps/slideshow.xpm")
		self.toolbar3.append_item(None, 'Slideshow', None, GtkPixmap(pix, mask), self.slideshow)

		self.scroll = GtkScrolledWindow()
		self.scroll.set_policy(POLICY_AUTOMATIC, POLICY_AUTOMATIC)
		self.scroll_adjust = self.scroll.get_vadjustment()
		self.vbox.pack_start(self.scroll)
		self.scroll.show()

		self.thumbs_container = GtkLabel()
		self.scroll.add_with_viewport(self.thumbs_container)
		self.thumbs_container.show()

	def select_dir(self, *args):
		'''Select the directory of images the user want to display.'''

		window = GtkWindow(WINDOW_TOPLEVEL)
		self.select_dir_window = window
		window.connect("delete_event", window.destroy)
		window.set_title("Sview - Select Directory")
		if screen_width() > 300:
			window.set_usize(280, 240)
		else:
			window.set_usize(240, 280)
		window.show()

		vbox = GtkVBox()
		window.add(vbox)
		vbox.show()

		scroll = GtkScrolledWindow()
		scroll.set_policy(POLICY_AUTOMATIC, POLICY_AUTOMATIC)
		vbox.pack_start(scroll)
		scroll.show()

		tree = GtkCTree(1, 0, ["Path"])
		self.dir_selection_tree = tree
		tree.connect("tree-select-row", self.tree_select)
		tree.connect("tree-expand", self.tree_expand)
		scroll.add_with_viewport(tree)
		tree.show()

		entry = GtkEntry()
		self.dir_entry = entry
		vbox.pack_start(entry, expand=FALSE)
		entry.show()

		hsep = GtkHSeparator()
		vbox.pack_start(hsep, expand=FALSE)
		hsep.show()

		hbutonbox = GtkHButtonBox()
		vbox.pack_start(hbutonbox, expand=FALSE)
		hbutonbox.show()

		button = GtkButton()
		button.connect("clicked", self.open_dir)
		hbutonbox.pack_start(button)
		button.show()

		button_hbox = GtkHBox()
		button.add(button_hbox)
		button_hbox.show()

		pix, mask = create_pixmap_from_xpm(self.window, None, "/usr/share/sview/pixmaps/open.xpm")
		button_pixmap = GtkPixmap(pix, mask)
		button_hbox.pack_start(button_pixmap)
		button_pixmap.show()

		button_label = GtkLabel("Open")
		button_hbox.pack_start(button_label)
		button_label.show()

		self.dir_open_pix, self.dir_open_mask = create_pixmap_from_xpm(self.window, None, "/usr/share/sview/pixmaps/dir-open.xpm")

		self.dir_close_pix, self.dir_close_mask = create_pixmap_from_xpm(self.window, None, "/usr/share/sview/pixmaps/dir-close.xpm")

		tree_item = self.dir_selection_tree.insert_node(None, None, ["/"], 5, self.dir_close_pix, self.dir_close_mask, self.dir_open_pix, self.dir_open_mask, FALSE, TRUE)
		self.dir_selection_tree.node_set_row_data(tree_item, { 'path': "/", 'blank_node': None })
		print tree_item

		self.list_dir(tree_item, "/")

	def list_dir(self, parent_tree_item, parent_dir):
		'''Add a list of directorys in dir to the tree.'''

		files = os.listdir(parent_dir)

		for file in files:
			full_path = parent_dir + "/" + file
			if os.path.isdir(full_path) == 1 and file[0] != ".":

				has_subdir = 0

				for sibling_file in os.listdir(full_path):
					if os.path.isdir(full_path + "/" + sibling_file) == 1 and sibling_file[0] != ".":
						has_subdir = 1
						break

				if has_subdir == 1:
					tree_item = self.dir_selection_tree.insert_node(parent_tree_item, None, [file], 5, self.dir_close_pix, self.dir_close_mask, self.dir_open_pix, self.dir_open_mask, FALSE, FALSE)
					blank_node = self.dir_selection_tree.insert_node(tree_item, None, [""])
				else:
					tree_item = self.dir_selection_tree.insert_node(parent_tree_item, None, [file], 5, self.dir_close_pix, self.dir_close_mask, self.dir_open_pix, self.dir_open_mask, TRUE, FALSE)
					blank_node = None

				self.dir_selection_tree.node_set_row_data(tree_item, { 'path': full_path, 'blank_node': blank_node })

	def tree_select(self, *args):
		'''Get the selected GtkCTreeNode's data.'''

		tree_item = args[1]
		node_data = self.dir_selection_tree.node_get_row_data(tree_item)
		self.dir_entry.set_text(node_data["path"][1:])

	def tree_expand(self, *args):
		'''Fill the parent GtkCTreeNode with siblings.'''

		tree_item = args[1]
		node_data = self.dir_selection_tree.node_get_row_data(tree_item)

		if node_data["blank_node"] != None:
			self.list_dir(tree_item, node_data["path"])

			self.dir_selection_tree.remove_node(node_data["blank_node"])
			node_data["blank_node"] = None
			self.dir_selection_tree.node_set_row_data(tree_item, node_data)

	def open_dir(self, *args):
		'''Check if the directory we want to read images from exists.'''

		dir = self.dir_entry.get_text()

		if os.path.isdir(dir) == 1:
			self.select_dir_window.destroy()
			self.toolbar3.set_sensitive(TRUE)
			self.images_dir = dir
			self.read_dir(dir)
		else:
			self.message_popup("Directory doesn't exist.")

	def refresh_dir(self, *args):
		'''Refresh the current directory.'''

		self.read_dir(self.images_dir)

	def read_dir(self, dir):
		'''Read the image files from a dir and load them into the gallery.'''

		self.image = []
		files = os.listdir(dir)

		for file in files:
			ext = file[-3:]
			if ext == "jpg" or ext == "gif" or ext == "png":
				self.image.append( { 'path': dir + "/" + file } )

		loading_window = GtkWindow(WINDOW_DIALOG)
		loading_window.connect("delete_event", loading_window.destroy)
		loading_window.set_title("Loading Gallery")
		loading_window.set_usize(130, 30)
		loading_window.show()

		vbox = GtkVBox()
		loading_window.add(vbox)
		vbox.show()

		loading_label = GtkLabel("Loading...")
		vbox.pack_start(loading_label, expand=FALSE)
		loading_label.show()

		adjust = GtkAdjustment(0, 0, len(self.image), 0, 0, 0)
		adjust_value = 0
		progress = GtkProgressBar(adjust)
		vbox.pack_start(progress, expand=FALSE)
		progress.show()

		while events_pending(): mainiteration()

		image_num = 0

		while image_num < len(self.image):

			gtk_image = gdkpixbuf.new_from_file(self.image[image_num]["path"])
			self.image[image_num]["orig_width"] = gtk_image.width
			self.image[image_num]["orig_height"] = gtk_image.height

			w = float(gtk_image.width)
			h = float(gtk_image.height)

			if w > 80 and w > h:
				k = float(w / h)
				w = 80.0
				h = float(w / k)
			elif h > 80:
				k = float(w / h)
				h = 80.0
				w = float(h * k)

			gtk_image = gtk_image.scale_simple(w, h, gdkpixbuf.INTERP_BILINEAR)
			self.image[image_num]["pixbuf"] = gtk_image

			adjust_value = adjust_value + 1
			progress.set_value(adjust_value)
			while events_pending(): mainiteration()

			image_num = image_num + 1

		if self.view_type == "list":
			self.show_list_view()
		else:
			self.show_thumbs_view()

		loading_window.destroy()

		while events_pending(): mainiteration()

	def show_list_view(self, *args):
		'''Display the images in the list of thumbs.'''

		image_num = 0

		self.show_main()
		self.view_type = 'list'
		self.thumbs_container.destroy()
		self.thumbs_container = GtkVBox()
		self.scroll.add_with_viewport(self.thumbs_container)
		self.thumbs_container.show()
		self.scroll_adjust.set_value(0)

		while image_num < len(self.image):

			hbox = GtkHBox()
			self.thumbs_container.pack_start(hbox, expand=FALSE)
			hbox.show()

			pix, mask = self.image[image_num]["pixbuf"].render_pixmap_and_mask()
			pixmap = GtkPixmap(pix, mask)
			pixmap.show()

			button = GtkButton()
			button.set_relief(RELIEF_NONE)
			button.connect("clicked", self.show_image, image_num)
			button.add(pixmap)
			hbox.pack_start(button, expand=FALSE)
			button.show()


			path = os.path.split(self.image[image_num]["path"])
			stats = os.stat(self.image[image_num]["path"])
			file_size = stats[6]

			if file_size < 1024:
				file_size_type = 'Bytes'
			else:
				file_size = file_size / 1024
				file_size_type = 'Kb'

			label = GtkLabel("Name: %s\nDimension: %sx%s\nSize: %s %s" % (path[len(path) - 1], self.image[image_num]["orig_width"], self.image[image_num]["orig_height"], file_size, file_size_type))
			label.set_justify(JUSTIFY_LEFT)
			hbox.pack_start(label, expand=FALSE)
			label.show()

			hsep = GtkHSeparator()
			self.thumbs_container.pack_start(hsep, expand=FALSE)
			hsep.show()

			image_num = image_num + 1

	def show_thumbs_view(self, *args):
		'''Display the images in a table of thumbs.'''

		cur_row_width = 0
		image_num = 0

		self.show_main()
		self.view_type = 'thumbs'
		self.thumbs_container.destroy()
		self.thumbs_container = GtkVBox()
		self.scroll.add_with_viewport(self.thumbs_container)
		self.thumbs_container.show()
		self.scroll_adjust.set_value(0)

		window_size = self.window.size_request()

		hbox = GtkHBox()
		self.thumbs_container.pack_start(hbox, expand=FALSE)
		hbox.show()

		while image_num < len(self.image):

			cur_row_width = cur_row_width + self.image[image_num]["pixbuf"].width

			if cur_row_width > window_size[0]:
				hbox = GtkHBox()
				self.thumbs_container.pack_start(hbox, expand=FALSE)
				hbox.show()

				cur_row_width = self.image[image_num]["pixbuf"].width

			pix, mask = self.image[image_num]["pixbuf"].render_pixmap_and_mask()
			pixmap = GtkPixmap(pix, mask)
			pixmap.show()

			button = GtkButton()
			button.set_relief(RELIEF_NONE)
			button.connect("clicked", self.show_image, image_num)
			button.add(pixmap)
			hbox.pack_start(button, expand=FALSE)
			button.show()

			image_num = image_num + 1

	def slideshow(self, *args):
		'''Either stop the current slideshow, or start a new one.'''

		if self.slideshow_on == 1:
			self.slideshow_end()
		else:
			self.slideshow_config()

	def slideshow_start(self, *args):
		'''Display the images in a slideshow.'''

		self.slideshow_on = 1

		image_delay = self.slideshow_spinbutton.get_value_as_int()
		self.slideshow_window.destroy()

		if self.view_type == 'image':
			self.close_image()
		self.show_image(None, 0)

		self.slideshow_timer = timeout_add(image_delay * 1000, self.next_image)

	def slideshow_end(self, *args):
		'''Stop the current slideshow.'''

		if self.slideshow_on == 1:
			self.slideshow_on = 0
			timeout_remove(self.slideshow_timer)
			self.show_main()
			self.message_popup("Slideshow stopped.")

	def slideshow_config(self, *args):
		'''Open the slideshow configuration window.'''

		window = GtkWindow(WINDOW_DIALOG)
		self.slideshow_window = window
		window.connect("delete_event", window.destroy)
		window.set_title("Sview - Slideshow")
		window.set_usize(200, 50)
		window.set_name("window")
		window.show()

		vbox = GtkVBox()
		window.add(vbox)
		vbox.show()

		hbox = GtkHBox()
		vbox.pack_start(hbox, expand=FALSE)
		hbox.show()

		label1 = GtkLabel("Delay ")
		hbox.pack_start(label1, expand=FALSE)
		label1.show()

		spin_adjust = GtkAdjustment(5, 0, 60, 1, 1, 1)

		spinbutton = GtkSpinButton()
		spinbutton.set_usize(40, -1)
		spinbutton.set_digits(0)
		spinbutton.set_adjustment(spin_adjust)
		spinbutton.set_value(5)
		hbox.pack_start(spinbutton, expand=FALSE)
		spinbutton.show()
		self.slideshow_spinbutton = spinbutton

		label2 = GtkLabel("seconds")
		hbox.pack_start(label2, expand=FALSE)
		label2.show()

		hsep = GtkHSeparator()
		vbox.pack_start(hsep, expand=FALSE)
		hsep.show()

		hbutonbox = GtkHButtonBox()
		vbox.pack_start(hbutonbox, expand=FALSE)
		hbutonbox.show()

		button = GtkButton()
		button.connect("clicked", self.slideshow_start)
		hbutonbox.pack_start(button)
		button.show()

		button_hbox = GtkHBox()
		button.add(button_hbox)
		button_hbox.show()

		pix, mask = create_pixmap_from_xpm(self.window, None, "/usr/share/sview/pixmaps/slideshow.xpm")
		button_pixmap = GtkPixmap(pix, mask)
		button_hbox.pack_start(button_pixmap)
		button_pixmap.show()

		button_label = GtkLabel("Start")
		button_hbox.pack_start(button_label)
		button_label.show()

	def scale_image(self, *args):
		'''Change the scale of the images.'''

		if self.image_scale == 'scaled':
			self.image_scale = 'none'
		else:
			self.image_scale = 'scaled'

		self.close_image()
		self.show_image(None, self.image_num)

	def show_image(self, *args):
		'''Show the next image.'''

		self.image_num = args[1]
		self.view_type = 'image'

		path = os.path.split(self.image[self.image_num]["path"])
		self.window.set_title("Sview " + path[len(path) - 1] + " - " + str(self.image_num + 1) + "/" + str(len(self.image)))

		self.toolbar2.set_sensitive(TRUE)

		self.scroll.hide()

		if self.image_scale == 'scaled':
			pixmap = self.render_image_scaled(self.image[self.image_num]["path"])
		else:
			pixmap = self.render_image_normal(self.image[self.image_num]["path"])

		self.view = GtkScrolledWindow()
		self.view.set_policy(POLICY_AUTOMATIC, POLICY_AUTOMATIC)
		self.view.add_with_viewport(pixmap)
		self.vbox.pack_start(self.view)
		self.view.show()

		self.window.connect("key-press-event", self.scroll_image)

	def scroll_image(self, widget, event):
		'''Called when the user hold a directional button.'''

		if event.keyval == 65361:
			change = -1
			adjust = self.view.get_hadjustment()
		elif event.keyval == 65363:
			change = 1
			adjust = self.view.get_hadjustment()
		elif event.keyval == 65362:
			change = -1
			adjust = self.view.get_vadjustment()
		elif event.keyval == 65364:
			change = 1
			adjust = self.view.get_vadjustment()
		else:
			change = 0
			adjust = self.view.get_vadjustment()

		value = adjust.value+(change*adjust.step_increment)
		adjust.set_value(max(0, min(value, adjust.upper-adjust.page_size)))

	def render_image_scaled(self, image):
		'''Render the pixbuf, scaling it to fit the screen. Then return a pixmap.'''

		gtk_image = gdkpixbuf.new_from_file(image)

		window_size = self.window.size_request()
		w = float(gtk_image.width)
		h = float(gtk_image.height)

		if w > 80 and w > h:
			k = float(w / h)
			w = float(window_size[0] - 10)
			h = float(w / k)
		elif h > 80:
			k = float(w / h)
			h = float(window_size[1] - 15)
			w = float(h * k)

		gtk_image = gtk_image.scale_simple(w, h, gdkpixbuf.INTERP_BILINEAR)

		pix, mask = gtk_image.render_pixmap_and_mask()
		pixmap = GtkPixmap(pix, mask)
		pixmap.show()

		return pixmap

	def render_image_normal(self, image):
		'''Render the pixbuf with no scaling. Then return a pixmap.'''

		gtk_image = gdkpixbuf.new_from_file(image)

		pix, mask = gtk_image.render_pixmap_and_mask()
		pixmap = GtkPixmap(pix, mask)
		pixmap.show()

		return pixmap

	def next_image(self, *args):
		'''Show the next image.'''

		if self.image_num < len(self.image) - 1:
			self.close_image()
			self.show_image(None, self.image_num + 1)
			return TRUE
		else:
			self.show_main()

	def prev_image(self, *args):
		'''Show the previous image.'''

		if self.image_num > 0:
			self.close_image()
			self.show_image(None, self.image_num - 1)
			return TRUE
		else:
			self.show_main()

	def close_image(self, *args):
		'''Close the current image.'''

		self.view.destroy()

	def show_main(self, *args):
		'''Close the current image and show the main view.'''

		if self.view_type == 'image':
			self.slideshow_end()
			self.close_image()
			self.window.set_title("Sview " + self.VERSION)
			self.scroll.show()
			self.toolbar2.set_sensitive(FALSE)

	def message_popup(self, message):
		'''Display a simple message popup.'''

		window = GtkWindow(WINDOW_DIALOG)
		window.connect("delete_event", window.destroy)
		window.set_title("Sview - Message")
		window.set_usize(200, 50)
		window.set_name("window")
		window.show()

		vbox = GtkVBox()
		window.add(vbox)
		vbox.show()

		label = GtkLabel(message)
		vbox.pack_start(label, expand=FALSE)
		label.show()

		hsep = GtkHSeparator()
		vbox.pack_start(hsep, expand=FALSE)
		hsep.show()

		hbutonbox = GtkHButtonBox()
		vbox.pack_start(hbutonbox, expand=FALSE)
		hbutonbox.show()

		button = GtkButton()
		button.connect("clicked", window.destroy)
		hbutonbox.pack_start(button)
		button.show()

		button_hbox = GtkHBox()
		button.add(button_hbox)
		button_hbox.show()

		pix, mask = create_pixmap_from_xpm(self.window, None, "/usr/share/sview/pixmaps/close.xpm")
		button_pixmap = GtkPixmap(pix, mask)
		button_hbox.pack_start(button_pixmap)
		button_pixmap.show()

		button_label = GtkLabel("Close")
		button_hbox.pack_start(button_label)
		button_label.show()

		return None

	def quit(self, *args):
		self.window.destroy()
		mainquit()

	def mainloop(self, *args):
		mainloop()

sview = Sview()
sview.mainloop()
